#! /usr/bin/env python
"""
openlog.py, Copyright(C) 2006, Sylvain Fourmanoit <syfou@users.sourceforge.net>

This code is released under the GPL, version 2. See `COPYING'.
"""
import sys, re, os
from os.path import *

#-------------------------------------------------------------------------------
# Test current interpreter version
#
if sys.hexversion < 0x20300f0:
  raise RuntimeError('This script needs at least a Python interpreter >= 2.3')

#-------------------------------------------------------------------------------
# Build the command line
#
from optparse import IndentedHelpFormatter,OptionParser

class MyFormatter(IndentedHelpFormatter):
  def format_usage(self, usage):
    return \
"""usage: %s [input_file]

Identify, test, sort and format syslog or dmesg entries generated by
openlog. Default input_file is standard input. Typical use for mksquasfs
sort file creation would be:

cat /path/to/log | openlog.py --root=/path/to/livecd > sort_file.txt
""" % usage

p = OptionParser(formatter=MyFormatter())
p.add_option('-r', '--root', dest='root', default='/',
             help=
"""root directory to prepend to file names (default is to use the real root).
This is performed once and for all before any other test, sort or output
formatting on filenames. This is chiefly useful when working with chrooted
environments""")
p.add_option('-o', '--output', dest='outfile', default=None,
             help='output file (default is standard output)')
p.add_option('-t', '--test', dest='test', default='isfile(@)',
             help=
"""test expression for file filtering. Any boolean-evaluable Python expression
will do, where the filename string will replace all occurence of the arobas
symbol. Default is `isfile(@)\', that only accepts existing regular files. Use
`True\' to perform no filtering at all, or `isdir(@) or isfile(@)\' to accept
both existing directories and regular files.  Alternatively, TEST can be the
path to an external script that should accept the filename as the only argument
and returns 0 to indicate selection; when no filename is given, the script must
also returns 0 (of course, this might be significantly slower than inline
expressions)""")
p.add_option('-k', '--keep-duplicate', dest='duplicate',
             action='store_true', default=False,
             help=
"""keep repeated files (default is not to). Please note that no sorting is
performed by this: original files ordering is preserved.""")
p.add_option('-s', '--sort-mode', dest='mode', default='reverse',
             help=
"""sort mode (default value is `reverse'). Valid values are `plain' (no sort,
files listed in order of occurence), `reverse' (files listed in inverse order of
occurence) and `alpha' (files listed by alphabetic order)""")
p.add_option('-f', '--format', dest='format', default='@\t#',
             help=
"""enumeration formatter (default is `@\\t#'). Specify the output format:
the arobas symbol is replaced by the filename, and the cardinal is replaced
by the rank of the file in the file list after an eventual sort. If you use
for instance `@', only the filename will be printed""")
p.add_option('-e', '--enum-start', dest='start', default='1',
             help=
"""rank of the first file (an integer, default is 1). It has no effect unless
the enumeration formatter uses ranking (see option --format)""")

#-------------------------------------------------------------------------------
# Parse the command line
#
opts, args = p.parse_args()

# Open input file
#
if len(args)>0 and args[0] != '-':
  try:
    opts.infile = file(args[0])
  except:
    p.error('could not open input file "%s"' % args[0])
else:
  opts.infile = sys.stdin

# Open output file
#
if opts.outfile is not None and opts.outfile != '-':
  try:
    opts.outfile = file(opts.outfile, 'w')
  except:
    p.error('could not open output file "%s" for writing' % opts.outfile)
else:
  opts.outfile = sys.stdout
  
# Check enumeration start rank
#
try:			      
  opts.start = int(opts.start) 
except ValueError:
  p.error('start rank "%s" is not an integer' % opts.start)

# Check sort mode
#
opts.mode = opts.mode.lower()
if not opts.mode in ('plain', 'reverse', 'alpha'):
  p.error('unknown sort mode `%s\'' % opts.mode)

# Construct and check test expression
#
if isfile(opts.test):	# First, check and test executable script
  opts.cmdline= '%s %%s > /dev/null 2>&1' % opts.test
  if os.system(opts.cmdline % '') == 0:
    opts.test = None

if opts.test is not None:
  opts.test = re.sub('@', '"%(file)s"', opts.test)
  try:
    if type(eval(opts.test % {'file': sys.argv[0]})) is not type(True):
      opts.test = False
  except:
    print 'Could not evaluate the given test operation, see the trace:'
    raise
if opts.test is False: p.error('test operation does not always return a boolean')

# Construct enumeration formatter expression
#
for pattern, repl in (('@', '%(file)s'), ('#', '%(rank)d')):
  opts.format = re.sub(pattern, repl, opts.format)

#-------------------------------------------------------------------------------
# Miscellaneous utilities: file tester and output formatter
#
class Tester(object):
  def __init__(self, opts): self.opts = opts
    
  def __call__(self, filename):
    if self.opts.test is not None:
      return eval(opts.test % {'file': filename})
    else:
      return os.system(self.opts.cmdline % filename) == 0
test = Tester(opts)

def enum(opts, iterable):
  i = opts.start
  for item in iterable:
    yield opts.format % {'file': item, 'rank': i}
    i += 1
    
#-------------------------------------------------------------------------------
# Perform the operation...
#
expr = re.compile('(^|.*\s)FILE\s+/?(.+)$')

# ... Fetch files, test them and remove duplicates as needed
#
files = []
for line in opts.infile.readlines():
  m = expr.match(line)
  if m is not None:
    f = join(opts.root, m.group(2))
    if (opts.duplicate or (not f in files)) and test(f):
      files.append(f)

# ... Sort the data
#
if opts.mode == 'reverse':
  files.reverse()
elif opts.mode == 'alpha':
  files.sort()
   
# ...And output the result
#
for line in enum(opts, files):
  print >> opts.outfile, line

#-------------------------------------------------------------------------------
# That's all folks!
#
